@@ -17,8 +17,8 @@
  *************************************************************************************/
 
 #include "analyzer.h"
-#include "localize.h"
-#include <kdemacros.h>
+
+#include <QCoreApplication>
 
 #include "operations.h"
 #include "value.h"
@@ -37,6 +37,16 @@
 #include "substituteexpression.h"
 #include "expressionstream.h"
 #include "matrix.h"
+#include "commands/listcommands.h"
+#include "commands/vectorcommands.h"
+#include "commands/matrixcommands.h"
+#include "commands/blockmatrixcommands.h"
+#include "commands/matrixqueries.h"
+
+#include "config-analitza.h"
+#ifdef HAVE_EIGEN3
+#include "commands/eigencommands.h"
+#endif
 
 // #define SCRIPT_PROFILER
 
@@ -125,6 +135,7 @@ Analyzer::Analyzer()
 	: m_vars(new Variables), m_varsOwned(true), m_hasdeps(true)
 {
 	m_runStack.reserve(defsize);
+	registerBuiltinMethods();
 }
 
 Analyzer::Analyzer(Variables* v)
@@ -132,6 +143,7 @@ Analyzer::Analyzer(Variables* v)
 {
 	m_runStack.reserve(defsize);
 	Q_ASSERT(v);
+	registerBuiltinMethods();
 }
 
 Analyzer::Analyzer(const Analyzer& a)
@@ -139,6 +151,7 @@ Analyzer::Analyzer(const Analyzer& a)
 {
 	m_vars = new Variables(*a.m_vars);
 	m_runStack.reserve(defsize);
+	registerBuiltinMethods();
 }
 
 Analyzer::~Analyzer()
@@ -147,12 +160,31 @@ Analyzer::~Analyzer()
 		delete m_vars;
 }
 
+void Analyzer::registerBuiltinMethods()
+{
+	m_builtin.insertFunction(RangeCommand::id, RangeCommand::type, new RangeCommand);
+	m_builtin.insertFunction(VectorCommand::id, VectorCommand::type, new VectorCommand);
+	m_builtin.insertFunction(MatrixCommand::id, MatrixCommand::type, new MatrixCommand);
+	m_builtin.insertFunction(BlockMatrixCommand::id, BlockMatrixCommand::type, new BlockMatrixCommand);
+	m_builtin.insertFunction(IdentityMatrixCommand::id, IdentityMatrixCommand::type, new IdentityMatrixCommand);
+	m_builtin.insertFunction(DiagonalMatrixCommand::id, DiagonalMatrixCommand::type, new DiagonalMatrixCommand);
+	m_builtin.insertFunction(BlockDiagonalMatrixCommand::id, BlockDiagonalMatrixCommand::type, new BlockDiagonalMatrixCommand);
+	m_builtin.insertFunction(TridiagonalMatrixCommand::id, TridiagonalMatrixCommand::type, new TridiagonalMatrixCommand);
+	m_builtin.insertFunction(IsZeroMatrixCommand::id, IsZeroMatrixCommand::type, new IsZeroMatrixCommand);
+	m_builtin.insertFunction(IsIdentityMatrixCommand::id, IsIdentityMatrixCommand::type, new IsIdentityMatrixCommand);
+	m_builtin.insertFunction(IsDiagonalMatrixCommand::id, IsDiagonalMatrixCommand::type, new IsDiagonalMatrixCommand);
+#ifdef HAVE_EIGEN3
+	m_builtin.insertFunction(EigenvaluesCommand::id, EigenvaluesCommand::type, new EigenvaluesCommand);
+	m_builtin.insertFunction(EigenvectorsCommand::id, EigenvectorsCommand::type, new EigenvectorsCommand);
+#endif
+}
+
 void Analyzer::setExpression(const Expression & e)
 {
 	m_exp=e;
 	flushErrors();
 	if(!e.tree()) {
-		m_err << i18n("Cannot calculate an empty expression");
+		m_err << QCoreApplication::tr("Cannot calculate an empty expression");
 	} else if(m_exp.isCorrect()) {
 		ExpressionTypeChecker check(m_vars);
 		check.initializeVars(m_builtin.varTypes());
@@ -194,7 +226,7 @@ Expression Analyzer::evaluate()
 		o=simp(o);
 		e.setTree(o);
 	} else {
-		m_err << i18n("Must specify a correct operation");
+		m_err << QCoreApplication::tr("Must specify a correct operation");
 	}
 	return e;
 }
@@ -209,11 +241,11 @@ Expression Analyzer::calculate()
 		e.setTree(calc(m_exp.tree()));
 	} else {
 		if(m_exp.isCorrect() && m_hasdeps)
-			m_err << i18n("Unknown identifier: '%1'",
+			m_err << QCoreApplication::tr("Unknown identifier: '%1'").arg(
 							dependencies(m_exp.tree(), m_vars->keys()+m_builtin.identifiers()).join(
-								i18nc("identifier separator in error message", "', '")));
+								QCoreApplication::translate("identifier separator in error message", "', '")));
 		else
-			m_err << i18n("Must specify a correct operation");
+			m_err << QCoreApplication::tr("Must specify a correct operation");
 	}
 	return e;
 }
@@ -222,7 +254,7 @@ Expression Analyzer::calculateLambda()
 {
 	Expression e;
 	
-	if(KDE_ISLIKELY(!m_hasdeps && m_exp.isCorrect())) {
+	if(Q_LIKELY(!m_hasdeps && m_exp.isCorrect())) {
 		Q_ASSERT(m_exp.tree()->isContainer());
 		Container* math=(Container*) m_exp.tree();
 		Q_ASSERT(math->m_params.first()->isContainer());
@@ -234,17 +266,17 @@ Expression Analyzer::calculateLambda()
 		Container* lambda=(Container*) math;
 		Q_ASSERT(lambda->containerType()==Container::lambda);
 		
-		if(KDE_ISUNLIKELY(m_runStack.first()!=lambda))
+		if(Q_UNLIKELY(m_runStack.first()!=lambda))
 			m_runStack.prepend(lambda);
 		m_runStackTop = 0;
 		e.setTree(calc(lambda->m_params.last()));
 	} else {
-		m_err << i18n("Must specify a correct operation");
+		m_err << QCoreApplication::tr("Must specify a correct operation");
 		
 		if(m_exp.isCorrect() && m_hasdeps)
-			m_err << i18n("Unknown identifier: '%1'",
+			m_err << QCoreApplication::tr("Unknown identifier: '%1'").arg(
 							dependencies(m_exp.tree(), m_vars->keys()).join(
-								i18nc("identifier separator in error message", "', '")));
+								QCoreApplication::translate("identifier separator in error message", "', '")));
 	}
 	return e;
 }
@@ -372,7 +404,7 @@ Object* Analyzer::eval(const Object* branch, bool resolve, const QSet<QString>&
 		Matrix::const_iterator it, itEnd=v->constEnd();
 		for(it=v->constBegin(); it!=itEnd; ++it) {
 			Object* res=eval(*it, resolve, unscoped);
-			nv->appendBranch(res);
+			nv->appendBranch(static_cast<MatrixRow*>(res));
 		}
 		ret=nv;
 	} else if(branch->type()==Object::apply) {
@@ -574,8 +606,8 @@ Object* Analyzer::calcPiecewise(const Container* c)
 		}
 	}
 	
-	if(KDE_ISUNLIKELY(!ret)) {
-		m_err << i18nc("Error message, no proper condition found.", "Could not find a proper choice for a condition statement.");
+	if(Q_UNLIKELY(!ret)) {
+		m_err << QCoreApplication::translate("Error message, no proper condition found.", "Could not find a proper choice for a condition statement.");
 		ret=new Cn(0.);
 	}
 	
@@ -609,14 +641,14 @@ Object* Analyzer::calcDeclare(const Container* c)
 	return ret;
 }
 
-template<class T, class Tit>
+template<class T, class Tit, class Tcontained>
 Object* Analyzer::calcElements(const Object* root, T* nv)
 {
 	const T *v=static_cast<const T*>(root);
 	Tit it, itEnd=v->constEnd();
 	
 	for(it=v->constBegin(); it!=itEnd; ++it)
-		nv->appendBranch(calc(*it));
+		nv->appendBranch(static_cast<Tcontained*>(calc(*it)));
 	
 	return nv;
 }
@@ -640,7 +672,7 @@ Object* Analyzer::calc(const Object* root)
 			ret = calcElements<List, List::const_iterator>(root, new List);
 			break;
 		case Object::matrix:
-			ret = calcElements<Matrix, Matrix::const_iterator>(root, new Matrix);
+			ret = calcElements<Matrix, Matrix::const_iterator, MatrixRow>(root, new Matrix);
 			break;
 		case Object::matrixrow:
 			ret = calcElements<MatrixRow, MatrixRow::const_iterator>(root, new MatrixRow);
@@ -720,11 +752,16 @@ Object* Analyzer::operate(const Apply* c)
 				for(; !stop && it!=itEnd; ++it) {
 					bool isValue = (*it)->type()==Object::value;
 					Object* v = isValue ? *it : calc(*it);
+					if(Q_UNLIKELY(v->isNone())) {
+						Q_ASSERT(!isValue);
+						ret = v;
+						break;
+					}
 					ret=Operations::reduce(opt, ret, v, &error);
 					if(!isValue)
 						delete v;
 					
-					if(KDE_ISUNLIKELY(error)) {
+					if(Q_UNLIKELY(error)) {
 						m_err.append(*error);
 						delete error;
 						error=0;
@@ -734,10 +771,13 @@ Object* Analyzer::operate(const Apply* c)
 					stop=isNull(opt, ret);
 				}
 			} else {
-				ret=Operations::reduceUnary(opt, calc(*c->firstValue()), &error);
-				if(KDE_ISUNLIKELY(error)) {
-					m_err.append(*error);
-					delete error;
+				ret = calc(*c->firstValue());
+				if(Q_LIKELY(!ret->isNone())) {
+					ret=Operations::reduceUnary(opt, ret, &error);
+					if(Q_UNLIKELY(error)) {
+						m_err.append(*error);
+						delete error;
+					}
 				}
 			}
 		}	break;
@@ -875,7 +915,7 @@ BoundingIterator* Analyzer::initBVarsContainer(const Analitza::Apply* n, int bas
 	switch(domain->type()) {
 		case Object::matrix:
 			Q_ASSERT(false && "fixme: properly iterate through elements when bounding");
-			if(static_cast<Matrix*>(domain)->size()>0)
+			if(static_cast<Matrix*>(domain)->rowCount()>0)
 				ret=new TypeBoundingIterator<Matrix, Matrix::const_iterator>(m_runStack, base, bvars, static_cast<Matrix*>(domain));
 			break;
 		case Object::list:
@@ -888,7 +928,7 @@ BoundingIterator* Analyzer::initBVarsContainer(const Analitza::Apply* n, int bas
 			break;
 		default:
 			Q_ASSERT(false && "Type not supported for bounding.");
-			m_err.append(i18n("Type not supported for bounding."));
+			m_err.append(QCoreApplication::tr("Type not supported for bounding."));
 	}
 	return ret;
 }
@@ -913,9 +953,9 @@ BoundingIterator* Analyzer::initBVarsRange(const Apply* n, int base, Object* obj
 			
 			ret=new RangeBoundingIterator(rr, u, d, 1);
 		} else
-			m_err.append(i18n("The downlimit is greater than the uplimit"));
+			m_err.append(QCoreApplication::tr("The downlimit is greater than the uplimit"));
 	} else
-		m_err.append(i18n("Incorrect uplimit or downlimit."));
+		m_err.append(QCoreApplication::tr("Incorrect uplimit or downlimit."));
 	return ret;
 }
 
@@ -936,7 +976,7 @@ Object* Analyzer::boundedOperation(const Apply& n, const Operator& t, Object* in
 		ret=Operations::reduce(type, ret, val, &correct);
 		delete val;
 		delete correct;
-	} while(KDE_ISLIKELY(it->hasNext() && !correct && !isNull(type, ret)));
+	} while(Q_LIKELY(it->hasNext() && !correct && !isNull(type, ret)));
 	
 	m_runStack.resize(top);
 	
@@ -1004,7 +1044,14 @@ Object* Analyzer::calcCallFunction(Container* function, const QVector<Object*>&
 		
 		m_runStack[top] = function;
 		for(int i=0; i<args.size(); i++) {
-			m_runStack[top+i+1] = args[i];
+			if (args[i]->type() != Object::none) {
+				m_runStack[top+i+1] = args[i];
+			} else {
+				m_err += QCoreApplication::tr("Invalid type for parameter '%1'").arg(i+1);
+				ret = new None();
+				
+				return ret;
+			}
 		}
 		m_runStackTop = top;
 		
@@ -1022,18 +1069,25 @@ Object* Analyzer::calcCallFunction(Container* function, const QVector<Object*>&
 		QList<Expression> expargs;
 		
 		for(int i=0; i<args.size(); i++) {
-			expargs += Expression(args[i]);
+			if (args[i]->type() != Object::none) {
+				expargs += Expression(args[i]);
+			} else {
+				m_err += QCoreApplication::tr("Invalid type for parameter '%1'").arg(i+1);
+				ret = new None();
+				
+				return ret;
+			}
 		}
 #ifdef SCRIPT_PROFILER
 		profiler.push(id);
 #endif
 		Expression exp=(*func)(expargs);
-		if(KDE_ISUNLIKELY(exp.isCorrect())) {
+		if(Q_UNLIKELY(exp.isCorrect())) {
 			ret=exp.tree();
 			exp.setTree(0);
 		} else {
 			m_err += exp.error();
-			ret = new Cn;
+			ret = new None();
 		}
 	}
 #ifdef SCRIPT_PROFILER
@@ -1119,13 +1173,13 @@ void Analyzer::simplify()
 	}
 }
 
-template <class T, class Tit>
+template <class T, class Tit, class Tcontained>
 void Analyzer::iterateAndSimp(T* v)
 {
 	Tit it = v->begin(), itEnd=v->end();
 	
 	for(; it!=itEnd; ++it)
-		*it = simp(*it);
+		*it = static_cast<Tcontained*>(simp(*it));
 }
 
 Object* Analyzer::simp(Object* root)
@@ -1147,7 +1201,7 @@ Object* Analyzer::simp(Object* root)
 	} else if(root->type()==Object::vector) {
 		iterateAndSimp<Vector, Vector::iterator>(static_cast<Vector*>(root));
 	} else if(root->type()==Object::matrix) {
-		iterateAndSimp<Matrix, Matrix::iterator>(static_cast<Matrix*>(root));
+		iterateAndSimp<Matrix, Matrix::iterator, MatrixRow>(static_cast<Matrix*>(root));
 	} else if(root->type()==Object::list) {
 		iterateAndSimp<List, List::iterator>(static_cast<List*>(root));
 	} else if(root->type()==Object::apply) {
@@ -1196,7 +1250,7 @@ bool actuallyE(const Object* o)
 QList<Transformation> simplifications()
 {
 	static QList<Transformation> ret;
-	if(KDE_ISUNLIKELY(ret.isEmpty())) {
+	if(Q_UNLIKELY(ret.isEmpty())) {
 		//divide
 		ret += Transformation(Transformation::parse("f/f"), Transformation::parse("1"));
 		ret += Transformation(Transformation::parse("f/1"), Transformation::parse("f"));
@@ -1327,7 +1381,7 @@ Object* Analyzer::simpApply(Apply* c)
 				c->m_params.last()=0;
 				QString* err=0;
 				val=Operations::reduceUnary(Operator::card, val, &err);
-				if(KDE_ISUNLIKELY(err)) { delete err; }
+				if(Q_UNLIKELY(err)) { delete err; }
 				delete c;
 				root=val;
 			}
@@ -1811,7 +1865,7 @@ bool Analyzer::insertVariable(const QString & name, const Object * value)
 {
 	bool wrong=!isLambda(value) && hasTheVar(QSet<QString>() << name, value);
 	if(wrong)
-		m_err << i18nc("By a cycle i mean a variable that depends on itself", "Defined a variable cycle");
+		m_err << QCoreApplication::translate("By a cycle i mean a variable that depends on itself", "Defined a variable cycle");
 	else
 		m_vars->modify(name, value);
 	
@@ -1853,7 +1907,7 @@ double Analyzer::derivative(const QVector<Object*>& values )
 		return 0.;
 	
 	if(!e1.isReal() || !e2.isReal()) {
-		m_err << i18n("The result is not a number");
+		m_err << QCoreApplication::tr("The result is not a number");
 		return 0;
 	}
 	
@@ -1883,7 +1937,7 @@ Object* Analyzer::applyAlpha(Object* o, int min)
 			case Object::container:	alphaConversion(static_cast<Container*>(o), min); break;
 			case Object::vector:	alphaConversion<Vector, Vector::iterator>(static_cast<Vector*>(o), min); break;
 			case Object::list:		alphaConversion<List, List::iterator>(static_cast<List*>(o), min); break;
-			case Object::matrix:	alphaConversion<Matrix, Matrix::iterator>(static_cast<Matrix*>(o), min); break;
+			case Object::matrix:	alphaConversion<Matrix, Matrix::iterator, MatrixRow>(static_cast<Matrix*>(o), min); break;
 			case Object::matrixrow:	alphaConversion<MatrixRow, MatrixRow::iterator>(static_cast<MatrixRow*>(o), min); break;
 			case Object::apply:		alphaConversion(static_cast<Apply*>(o), min); break;
 			case Object::variable: {
@@ -1907,13 +1961,13 @@ Object* Analyzer::applyAlpha(Object* o, int min)
 	return o;
 }
 
-template <class T, class Tit>
+template <class T, class Tit, class Tcontained>
 void Analyzer::alphaConversion(T* o, int min)
 {
 	Q_ASSERT(o);
 	Tit it=o->begin(), itEnd=o->end();
 	for(; it!=itEnd; ++it) {
-		*it = applyAlpha(*it, min);
+		*it = static_cast<Tcontained*>(applyAlpha(*it, min));
 	}
 }
 
